/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package abp.usage.dist;

import static abp.usage.dist.Configuration.PORT_ABP;
import static abp.usage.dist.Configuration.PORT_ACK;
import static abp.usage.dist.Configuration.SENDER_HOST;

import java.util.Observable;
import java.util.Observer;

import sim.IScheduler;
import sim.error.SimpleErrorHandler;
import sim.port.IPortFactory;
import sim.port.TimedObservablePort;
import sim.remote.IInTCPPort;
import sim.remote.TCPPort;
import sim.remote.TCPPortFactory;
import sim.sched.SchedulerFactory;
import abp.ABPMessage;
import abp.gen.factories.ABPReceiverFactory;
import abp.gen.interfaces.IABPReceiver;

/**
 * Configures the receiver part of the ABP for a distributed simulation.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3056 $, 
 *   $Date: 2014-08-06 18:21:22 +0200 (Mi, 06 Aug
 * 2014) $
 * @since 2.5.0
 */
public class RemoteAbpReceiver {
  
  private IABPReceiver<String> receiver;
  //->RemoteAbpReceiver.setUp.java<
  /**
   * Configures an ABPReceiver for a distributed simulation.
   * @param addr IP address of the ABP sender. 
   */
  public void setUp(String addr) {
    receiver = ABPReceiverFactory.create();

    // configure scheduler to create TCPPorts
    IScheduler sched = SchedulerFactory.createDefaultScheduler();
    IPortFactory factory = new TCPPortFactory();
    sched.setPortFactory(factory);
    receiver.setup(sched, new SimpleErrorHandler());
    
    // configure ack out port
    TCPPort<Boolean> ackOut = new TCPPort<Boolean>();
    ackOut.addReceiver(addr, PORT_ACK);
    receiver.setAck(ackOut);
    
    // configure and get abp in port
    IInTCPPort<ABPMessage<String>> abpMsg = 
        (IInTCPPort<ABPMessage<String>>) receiver
        .getAbpMessage();
    abpMsg.startListenOn(PORT_ABP);
    
    // configure message out port
    TimedObservablePort<String> results = 
        new TimedObservablePort<String>();
    // Use 'result' as outgoing port message. 
    receiver.setMessage(results);
    // observer is updated, if a message is received 
    results.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        System.out.println("Received: " + arg);
      }
    });
  }
  //->RemoteAbpReceiver.setUp.java<
  
  /**
   * Main method to start a distributed receiver.
   * @param args not used
   */
  public static void main(String[] args) {
      String addr;
      if (args.length >= 2) {
          addr = args[1];
      }
      else {
          addr = SENDER_HOST;
      }
    RemoteAbpReceiver rar = new RemoteAbpReceiver();
    rar.setUp(addr);
  }
}
