/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package abp.usage.dist;

import static abp.usage.dist.Configuration.PORT_ABP;
import static abp.usage.dist.Configuration.PORT_ACK;
import static abp.usage.dist.Configuration.RECEIVER_HOST;

import java.util.Scanner;

import sim.IScheduler;
import sim.error.SimpleErrorHandler;
import sim.generic.Tick;
import sim.port.IPortFactory;
import sim.remote.IInTCPPort;
import sim.remote.TCPPort;
import sim.remote.TCPPortFactory;
import sim.sched.SchedulerFactory;
import abp.ABPMessage;
import abp.usage.dist.gen.factories.DelayedSenderFactory;
import abp.usage.dist.gen.interfaces.IDelayedSender;
/**
 * Configures the sender part of the ABP for a distributed 
 * simulation.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3146 $, 
 *          $Date: 2015-06-27 13:31:24 +0000 (Sat, 27 Jun 2015) $
 * @since 2.5.0
 */
//->RemoteAbpSender.java<
public final class RemoteAbpSender implements Runnable {
  
  private IDelayedSender<String> sender;
  private static final Tick<String> TICK = Tick.<String> get();
  
  /**
   * Configures a DelayedSender for a distributed simulation.
   * @param addr IP address of the ABP receiver.
   */
  public void setUp(String addr) {
    sender = DelayedSenderFactory.create(2, 1);
    
    // configure scheduler to use TCPPortFactory
    IScheduler sched = SchedulerFactory.createDefaultScheduler();
    IPortFactory factory = new TCPPortFactory();
    sched.setPortFactory(factory);
    sender.setup(sched, new SimpleErrorHandler());
    
    // configure abp out port
    TCPPort<ABPMessage<String>> abpOut = 
        new TCPPort<ABPMessage<String>>();
    abpOut.addReceiver(addr, PORT_ABP);
    sender.setAbpMessage(abpOut);
    
    // Get and configure ack in port
    IInTCPPort<Boolean> ack = (IInTCPPort<Boolean>) 
        sender.getAck();
    ack.startListenOn(PORT_ACK);
  }
  
  /**
   * Sends the given message to port message.
   * @param msg message to send
   */
  public void sendMessage(String msg) {
    System.out.println("Sending: " + msg);
    sender.getMessage().accept(msg);
  }
  
  /**
   * Sends a tick to port message.
   */
  public void sendTick() {
    sender.getMessage().accept(TICK);
  }
  
  /**
   * Main method to start a distributed sender.
   * 
   * @param args not used
   */
  public static void main(String[] args) {
      String addr;
      if (args.length >= 2) {
          addr = args[1];
      }
      else {
          addr = RECEIVER_HOST;
      }
    final RemoteAbpSender ras = new RemoteAbpSender();
    ras.setUp(addr);
    ras.run();
  }

  @Override
  public void run() {
    // periodically triggers ras to send a tick
    Ticker ticker = new Ticker(this);
    new Thread(ticker).start();
    
    // scanner is used to read input from console
    Scanner scanner = new Scanner(System.in);
    try {
        boolean running = true;
        while (running) {
            String input = scanner.next();
            if ("exit".equals(input)) {
                running = false;
            }
            this.sendMessage(input);
        }        
    } 
    finally {
        // close scanner and stop threads
        scanner.close();
        ticker.stop();
        ((TCPPort<ABPMessage<String>>) sender.getAbpMessage()).stop();
        ((TCPPort<Boolean>) sender.getAck()).stop();
        ((TCPPort<String>) sender.getMessage()).stop();
    }
  }  
}
//->RemoteAbpSender.java<
