/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package abp.usage.dist;

/**
 * Periodically triggers the sender to send a tick.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 2953 $, $Date: 2014-08-06 18:21:22 +0200 (Mi, 06 Aug
 * 2014) $
 * @since 2.5.0
 */
class Ticker implements Runnable {
  /** True, if the ticker is running. */
  private boolean isRunning = true;
  
  /** Used {@link RemoteAbpSender}. */
  private final RemoteAbpSender sender;
  
  /** Delta between two ticks in ms. */
  private static final int SLEEP_TIME = 500;
  
  /**
   * Creates a new {@link Ticker} that triggers the 
   * given {@link RemoteAbpSender}.
   * @param ras triggered {@link RemoteAbpSender}. 
   */
  public Ticker(final RemoteAbpSender ras) {
    this.sender = ras;
  }
  
  /**
   * Stops the {@link Ticker}.
   */
  public void stop() {
    isRunning = false;
  }
  
  @Override
  public void run() {
    while (isRunning) {
      sender.sendTick();
      try {
        Thread.sleep(SLEEP_TIME);
      }
      catch (InterruptedException e) {
        // ignore
      }
    }
    System.out.println("Ticker stopped");
  }
}
